@@ -991,6 +991,13 @@ winMultiWindowWMProc(void *pArg)
             /* Determine the Window style, which determines borders and clipping region... */
             UpdateStyle(pWMInfo, pNode->msg.iWindow, &maxmin);
 
+            /* Make sure it gets the proper system menu for a WS_POPUP, too */
+            GetSystemMenu(pNode->msg.hwndWindow, TRUE);
+
+#define WM_INIT_SYS_MENU	(WM_USER + 1001)
+            /* Cause any .XWinrc menus to be added in main WNDPROC */
+            PostMessage(pNode->msg.hwndWindow, WM_INIT_SYS_MENU, 0, 0);
+
             /* Display the window without activating it */
             {
                 xcb_get_window_attributes_cookie_t cookie;
@@ -1109,6 +1116,7 @@ winMultiWindowWMProc(void *pArg)
                -- independently, the WM_TAKE_FOCUS protocol determines whether
                the WM should send a WM_TAKE_FOCUS ClientMessage.
             */
+            if (pNode->msg.iWindow)
             {
               Bool neverFocus = FALSE;
               xcb_get_property_cookie_t cookie;
@@ -1122,7 +1130,7 @@ winMultiWindowWMProc(void *pArg)
               }
 
               if (!neverFocus)
-                xcb_set_input_focus(pWMInfo->conn, XCB_INPUT_FOCUS_POINTER_ROOT,
+                xcb_set_input_focus(pWMInfo->conn, XCB_INPUT_FOCUS_PARENT,
                                     pNode->msg.iWindow, XCB_CURRENT_TIME);
 
               if (IsWmProtocolAvailable(pWMInfo,
@@ -1132,6 +1140,13 @@ winMultiWindowWMProc(void *pArg)
                              pNode->msg.iWindow,
                              pWMInfo->atmWmProtos, pWMInfo->atmWmTakeFocus);
 
+            }
+            else
+            /* Set the input focus to none */
+            {
+              xcb_set_input_focus(pWMInfo->conn, XCB_INPUT_FOCUS_NONE,
+                                  XCB_INPUT_FOCUS_NONE, XCB_CURRENT_TIME);
+
             }
             break;
 
@@ -2169,6 +2184,7 @@ winApplyHints(WMInfoPtr pWMInfo, xcb_window_t iWindow, HWND hWnd, HWND * zstyle,
                     || (size_hints.max_height < GetSystemMetrics(SM_CYVIRTUALSCREEN)))
                     hint |= HINT_NOMAXIMIZE;
 
+
                 if (size_hints.flags & XCB_ICCCM_SIZE_HINT_P_MIN_SIZE) {
                     /*
                        If both minimum size and maximum size are specified and are the same,
